package com.server;
import java.io.*;
import java.security.*;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import javax.crypto.*;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

/** pomoćna klasa koja se koristi za enkripciju i dekripciju
 *   podataka
 */


public class Security {

	private static Key publicKey=null;
	private static Key privateKey=null;
	private byte[] AESkeyb;
	
	// konstruktor klase Security
	
	public Security() {
		
		try {
			generateKeyPair();              // konstruktor poziva metodu generateKeyPair() koja generiše ključeve
		} catch (Exception e) {

			e.printStackTrace();
		}
		
	}
	// metoda za generisanje javnog i tajnog RSA ključa
	
	public void generateKeyPair() throws Exception {
		
		KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");  // kreira generator para ključeva
		SecureRandom random = new SecureRandom();                       // kreira generator niza slučajnih brojeva   
		
		keyGen.initialize(1024, random);                                 // dodeljuje generatoru ključeva, generator slučajnih brojeva, i postavlja
                                                                         // dužinu ključeva na 1024 bita  
		KeyPair generatedKeyPair = keyGen.genKeyPair();                  // generiše par ključeva i čuva ih u promenljivu generatedKeyPair
         
		publicKey = generatedKeyPair.getPublic();                        // generatedKeyPair.getPublic vraća vrednost javnog ključa  
		privateKey = generatedKeyPair.getPrivate();                      // generatedKeyPair.getPrivate vraća vrednost privatnog ključa  
		
		
		
		}
	         // metoda vraća vrednost javnog RSA ključa	
	
	    public String getPublicKey (){
			
			String pubKeyStr = Base64.encodeToString(publicKey.getEncoded(), 0);
		
			System.out.print("Public key:\n");
			System.out.print(pubKeyStr+"\n");
			
	        return pubKeyStr;
	    	
	    }
	    // metoda vraća vrednost tajnog RSA ključa
	    
	    public static Key getPrivateKey (){
	    	
	    	return privateKey;
	    }
	
	    
	    // metoda za dekripciju RSA algoritmom 
	    
	    	public void decryptRSA(String text) throws Exception {

	    	Cipher cipher=null;

	    	byte[] decryptedData = null;
	    	
	    	byte[] data = Base64.decode(text, 0);

	    	cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");       // kreira šifrat za RSA algoritam

	    	cipher.init(Cipher.DECRYPT_MODE, getPrivateKey());         //inicira mod za dekripciju i povlači vrednost RSA tajnog ključa 
                                                                      //pomoću metode getPrivateKey

	    	decryptedData = cipher.doFinal(data);                   // doFinal izvršava dekripciju RSA algoritmom
                                                                   // podataka smeštenih u promenljivoj data 	    	
	    	AESkeyb = decryptedData;                                // dekriptovani podaci se dodeljuju promenljivoj AESkeyb, koja označava AES ključ
	    	
	    	System.out.print("AES key:\n");
			System.out.print(Base64.encodeToString(AESkeyb, 0)+"\n");

	    	}

	    	// ova metoda vrši enkripciju podataka AES ključem         

	    		 public String encrypt(String text) throws Exception {
	    			 
	    			 
	    			 Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");             //kreira sifrat za AES algoritam, u CBC modu i prema PKCS5 standardu                             
	    			    byte[] keyBytes = new byte[16];                                      //definiše niz od 16 bajtova 
	    			    int len = AESkeyb.length;                                            // definiše dužinu AES ključa
	    			    if (len > keyBytes.length)                                           // ukoliko je dužina AES ključa duža od predviđenih 16 bajtova     
	    			        len = keyBytes.length;                                           // ograničava je
	    			    System.arraycopy(AESkeyb, 0, keyBytes, 0, len);                      // funkcija System.arraycopy kopira vrednost AES ključa u novu promenljivu keyBytes 
	    			    SecretKeySpec keySpec = new SecretKeySpec(keyBytes, "AES");          //definiše specifikaciju za AES ključ
	    			    IvParameterSpec ivSpec = new IvParameterSpec(keyBytes);              // definiše vektor za inicijalizaciju
	    			    cipher.init(Cipher.ENCRYPT_MODE, keySpec, ivSpec);                   // inicira mod enkripcije i dodeljuje mu keySpec i ivSpec promenljive

	    			    byte[] results = cipher.doFinal(text.getBytes("UTF-8"));             // završava enkripciju  
	    			    
	    			    return Base64.encodeToString(results, 0);
	    			 
	    		 }
	    			 
	    		// ova metoda vrši dekripciju podataka AES ključem
	    		 
	    		 public String decrypt(String data) throws Exception {
	    			 
	    			 Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");             //kreira sifrat za AES algoritam     
	    			    byte[] keyBytes = new byte[16];
	    			    
	    			    int len = AESkeyb.length;                                            // definiše dužinu AES ključa
	    			    if (len > keyBytes.length)
	    			        len = keyBytes.length;
	    			    System.arraycopy(AESkeyb, 0, keyBytes, 0, len);
	    			    SecretKeySpec keySpec = new SecretKeySpec(keyBytes, "AES");
	    			    IvParameterSpec ivSpec = new IvParameterSpec(keyBytes);
	    			    cipher.init(Cipher.DECRYPT_MODE, keySpec,ivSpec);                   // inicira mod dekripcije i dodeljuje mu keySpec i ivSpec promenljive 

	    			    byte[] results = cipher.doFinal(Base64.decode(data, 0));            // završava dekripciju
	    			    return new String(results, "UTF-8");

	    			 }

}